(*========================================================*)
(*                          IPX.PAS                       *)
(*               (c) 1993 G. Blumert & DMV-Verlag         *)
(*--------------------------------------------------------*)
(* Compiler:                                              *)
(*   Turbo-Pascal ab 6.0 oder Turbo-Pascal fr Windows    *)
(*                                                        *)
(* Funktion:                                              *)
(*   Implementiert die fr die Kommunikation mit IPX      *)
(*   ntigen Routinen                                     *)
(*========================================================*)

UNIT IPX;
{X+,B-,S-}
(* $S- ist wichtig, da einige Routinen vom Event-Handler  *)
(* aufgerufen werden                                      *)

INTERFACE

USES
{$IFDEF Windows}
  WinDos, {$IFDEF Ver70} Objects, OWindows, {$ELSE}
  WObjects, {$ENDIF} WinProcs, WinTypes, DPMI;
{$ELSE}                                   (* Oh-Borland! *)
  Dos, Objects, Views, Memory, App, MsgBox;
{$ENDIF}

TYPE
  (* Knoten-Adresse: 6 Bytes: *)
  tNodeAdr = ARRAY[0..5] OF BYTE;

  (* Rechner-ID - NetAdr ist 0 fr das gleiche Netz: *)
  pIPX_ID = ^tIPX_ID;
  tIPX_ID = RECORD
    NetAdr : LongInt;
    NodeAdr: tNodeAdr;
  END;

  pUserData = ^tUserData;
  tUserData = RECORD
  (* Hier mssen die eigenen Datendefinitionen hin *)
  (* Dieser Record wird an tIPX_Data angehngt     *)
  END;

(* Datentyp, der von IPX bertragen werden soll      *)
(* Maximalgre lt. c't 7'91 576 Bytes - ob mit oder *)
(* ohne Header mu selbst ausprobiert werden.        *)
  pIPX_Data = ^tIPX_Data;
  tIPX_Data = RECORD
    SourceSocket: WORD;
{$IFDEF Windows}
    SourceWin : pWindowsObject;
    (* Einfacher wre, TYPE pView = pWindowsObject *)
    DestWin   : pWindowsObject;
    (* Aber ich denke, so herrscht mehr bersicht  *)
{$ELSE}
    SourceWin : pView;
    DestWin   : pView;
{$ENDIF}
    Command   : WORD;
    (* Derzeit noch leer - fr eigene Daten: *)
    UserData  : tUserData;        
  END;

  (* Struktur des Datenpakets                             *)
  (* Fr den Empfang eines Paketes mu nichts             *)
  (* initialisiert werden                                 *)
  pIPX_Packet = ^tIPX_Packet;
  tIPX_Packet = RECORD
    CheckSum        : WORD; (* Wird von IPX genutzt       *)
    Length          : WORD; (* Wird von IPX gesetzt       *)
    TransportControl: BYTE; (* Wird von IPX genutzt       *)
    PacketType      : BYTE; (* 0: Unknown;
                               1: Routing Information
                                  Packet;
                               2: Echo Packet;
                               3: Error Packet;
                               4: PacketExchange Packet;
                               5: Sequenced Packet Protocol
                           16-31: Experimental Protocols  *)
    Dest            : tIPX_ID; (* Zieladresse             *)
    Dest_Socket     : WORD;    (* Ziel-Socket             *)
    Source          : tIPX_ID; (* Quelladresse - wird von *)
                               (* IPX eingesetzt          *)
    Source_Socket   : WORD;
(* Direkt auf diese Struktur folgen die Daten; der Header *)
(* wird im Feld Length mitgerechnet                       *)
    Data            : tIPX_Data;
  END;


TYPE
  pECB = ^tECB;              (* Event - Control - Block   *)
  tECB = RECORD
    LinkAdr   : POINTER;     (* Wird von IPX verwendet    *)
    ESR_Adr   : POINTER;     (* Adresse des Event-        *)
                             (* Handlers - NIL = ungltig *)
    InUseFlag : BYTE;        (* Wird von IPX verwendet    *)
    ComplCode : BYTE;        (* Wird von IPX verwendet    *)
    Socket    : WORD;        (* Socket-Nummer             *)
    IPX_Work  : LongInt;     (* Wird von IPX verwendet    *)
    Drv_Work  : ARRAY[0..11] OF BYTE; (*    "             *)
    Imm_Adr   : tNodeAdr;    (* Knoten-Adresse            *)
    Frag_Count: WORD;        (* Anzahl der Daten-Pakete   *)
    Frag_Adr  : pIPX_Packet; (* Adr. d. Datenpaketes      *)
    Frag_Size : WORD;        (* Gre d. Datenpaketes     *)
  END;

CONST

(* Eigene Adresse - wird von InitIPX ausgefllt         *)
  MyIPX_ID: tIPX_ID = (NetAdr: 0; NodeAdr: (0,0,0,0,0,0));

(* Ergebnisse der Funktion OpenSocket:                    *)
  osm_SocketOk          = 0;   (* Socket wie gewnscht    *)
                               (* geffnet                *)
  osm_SocketAlreadyOpen = $FF; (* Socket ist bereits      *)
                               (* geffnet                *)
  osm_NoMoreSockets     = $FE; (* Kein freies Socket mehr *)

(* ffnungs-Modus fr OpenSocket                          *)
(* Geffnet bleiben bis zum Schlieen oder Programmende   *)
(* Automatisches Schlieen funktioniert unter Netware-    *)
(* Lite nicht                                             *)
  os_AutoClose = 0;
(* Geffnet bleiben bis zum Schlieen. Fr residente      *)
(* Programme                                              *)
  os_StayOpen  = $FF;

(* Ergebnisse der Funktion CancelEvent:                   *)
  cem_Ok          = 0;           (* Alles klar!           *)
  cem_NotPossible = $F9;         (* Lschen nicht mglich *)
  cem_InvalidECB  = $FF;         (* ECB existiert nicht   *)

{$IFDEF Windows}
  wm_IPXFirst       = wm_User+10;
{$ELSE}
  wm_IPXFirst       = 1034;
{$ENDIF}
  wm_LoggedIn       =  wm_IPXFirst + 0;
  wm_LoggedOut      =  wm_IPXFirst + 1;
  wm_CommandReceived = wm_IPXFirst + 2;

(* Stellt fest, ob IPX geladen ist und speichert die      *)
(* Einsprungadresse in IPX_Adr.                           *)
(* Unter Windows wird obendrein noch die Call-Back-       *)
(* Adresse fr den Event-Handler mit WinNet ausgetauscht  *)
(* -- siehe Dokumentation                                 *)
{$IFDEF Windows}
FUNCTION InitIPX(DOSAdr: POINTER): POINTER;
{$ELSE}
FUNCTION InitIPX: BOOLEAN;
{$ENDIF}

(* Ermittelt Netz- und Knotenadresse des Rechners         *)
PROCEDURE GetInternetworkAddress(Buffer: pIPX_ID);

(* Versucht, das angegebene Socket zu ffnen.             *)
(* Wird versucht, Socket 0 zu ffnen, so vergibt IPX eine *)
(* freie Nummer.                                          *)
(* Die Nummer des geffneten Sockets wird zurckgegeben.  *)
(* Result siehe osm_xxx - Konstanten                      *)
(* Modus  siehe os_xxx - Konstanten                       *)
PROCEDURE OpenSocket(VAR Socket: WORD;
                     VAR Result: BYTE; Modus: BYTE);

(* Schliet das angegebene Socket *)
PROCEDURE CloseSocket(Socket: WORD);

(* Stoppt die Bearbeitung des angegebenen ECB. Ist        *)
(* erforderlich, um die Funktion ListenForPacket zu       *)
(* stoppen. Ergebnis s. cem_XXX - Konst.                  *)
FUNCTION CancelEvent(AnECB: pECB): BYTE;

(* Lauscht nach Datenpaketen                              *)
PROCEDURE ListenForPacket(AnECB: pECB);

(* Schickt ein Paket ab und ffnet/schliet dafr ein     *)
(* Socket.                                                *)
(* Ergebnis siehe osm_xxx - Konstanten.                   *)
(* Unter Windows mssen zustzlich die Selektoren von     *)
(* Packet und ECB mit bergeben werden, damit die Angaben *)
(* ber den Socket eingetragen werden  knnen.            *)
(* AnECB mu die Adresse im Real-Mode-Format              *)
(* Segment:Offset enthalten                               *)
FUNCTION SendPacketWithoutSocket(AnECB: pECB
         {$IFDEF Windows}; pModeECB: pECB {$ENDIF})
                                                  : INTEGER;

(* Schickt ein Paket ab und geht davon aus, da das       *)
(* Socket bereits geffnet und korrekt im ECB eingetragen *)
(* ist.                                                   *)
(* Wird von SendPacketWithoutECB aufgerufen.              *)
(* AnECB mu in jedem Fall aus Real-Mode Segment:Offset   *)
(* bestehen.                                              *)
PROCEDURE SendPacketWithSocket(AnECB: pECB);

(* Sollte nach dem Abschicken eines Paketes aufgerufen    *)
(* werden, wenn das Socket noch fr die                   *)
(* Empfangsbesttigung offen bleiben soll.                *)
(* Sorgt dafr, da IPX Zeit bekommt, das Ereignis zu     *)
(* verarbeiten.                                           *)
PROCEDURE RelinquishControl;

IMPLEMENTATION

{$IFDEF Windows}
FUNCTION InitIPX(DOSAdr: POINTER): POINTER;
VAR
  DosMem   : LongInt;
  p        : pIPX_ID;
  DOSReturn: POINTER;
BEGIN
  DOSReturn := InitIPXInt(DOSAdr);
  InitIPX   := DOSReturn;
  IF DOSReturn <> NIL THEN BEGIN
                                (* DOS-Speicher anfordern *)
    DosMem := GlobalDosAlloc(SizeOf(MyIPX_ID));
                              (* Segment - fr Real-Mode: *)
    p := Ptr(HiWord(DosMem), 0);
    GetInternetworkAddress(p);
    p := Ptr(LoWord(DosMem), 0);            (* Selektor   *)
    MyIPX_ID := p^;                         (* Umkopieren *)
    GlobalDosFree(LoWord(DosMem))           (* freigeben  *)
  END
END;

{$ELSE}

CONST
  IPX_Adr: POINTER = NIL;                (* IPX-Einsprung *)

FUNCTION InitIPX: BOOLEAN;
BEGIN
  IF IPX_Adr <> NIL THEN InitIPX := TRUE ELSE BEGIN
    ASM
      MOV AX, 7A00H
      Int 2FH
      CMP AL, 0FFH
      JNE @Ende
      MOV WORD Ptr [IPX_Adr], DI  (* IPX-Einsprung merken *)
      MOV WORD Ptr [IPX_Adr+2], ES
@Ende:
    END;
    IF IPX_Adr <> NIL THEN BEGIN
      InitIPX := TRUE;       (* Netz- und Knoten-Adresse: *)
      GetInternetworkAddress(@MyIPX_ID);
    END ELSE InitIPX := FALSE
  END
END;

{$ENDIF}

{$IFDEF Windows}
PROCEDURE GetInternetworkAddress(Buffer: pIPX_ID);
BEGIN
  FillStruc(CallStrc,Buffer,0,9,0,0);
  RMInt(CallStrc);
END;

{$ELSE}

PROCEDURE GetInternetworkAddress(Buffer: pIPX_ID);
                                                  ASSEMBLER;
ASM
  LES  SI, Buffer
  MOV  BX, 9                              (* Funktion $09 *)
  CALL DWORD Ptr IPX_Adr
END;
{$ENDIF}


PROCEDURE OpenSocket(VAR Socket: WORD;
                     VAR Result: BYTE; Modus: BYTE);
{$IFDEF Windows}
BEGIN
  FillStruc(CallStrc,NIL,WORD(Modus),0,0,Socket);
  RMInt(CallStrc);
  Result := CallStrc^.EAX AND $FF;
  Socket := CallStrc^.EDX AND $FFFF

{$ELSE}
                                                  ASSEMBLER;
ASM
  LES  SI, Socket
  MOV  DX, ES:[SI]
  MOV  AL, Modus
  XOR  BX, BX                             (* Funktion $00 *)
  CALL DWORD Ptr IPX_Adr
  MOV  ES:[SI], DX
  LES  SI, Result
  MOV  ES:[SI], AL
{$ENDIF}
END;

PROCEDURE CloseSocket(Socket: WORD);
{$IFDEF Windows}
BEGIN
  FillStruc(CallStrc, NIL, 0, 1, 0, Socket);
  RMInt(CallStrc);

{$ELSE}
                                                  ASSEMBLER;
ASM
  MOV   DX, Socket
  MOV   BX, 1                           (* Funktion $01   *)
  CALL  DWORD Ptr IPX_Adr               (* Keine Rckgabe *)
{$ENDIF}
END;

PROCEDURE SendPacketWithSocket(AnECB: pECB);
{$IFDEF Windows}
BEGIN
  FillStruc(CallStrc, AnECB, 0, 3, 0, 0);
  RMInt(CallStrc)

{$ELSE}
                                                  ASSEMBLER;
ASM
  LES  SI, AnECB
  MOV  BX, 3                            (* Funktion $03   *)
  CALL DWORD Ptr IPX_Adr                (* Keine Rckgabe *)
{$ENDIF}
END;

FUNCTION SendPacketWithoutSocket(AnECB: pECB
{$IFDEF Windows} ; pModeECB: pECB {$ENDIF}): INTEGER;

VAR                  (* Result siehe osm_xxx - Konstanten *)
  osResult     : BYTE;
  ASocket      :  WORD;
  h, m, s, s100: WORD;
  l            : LongInt;
  Versuch      : INTEGER;

BEGIN
  OpenSocket(ASocket, osResult, os_AutoClose);
  SendPacketWithoutSocket := osResult;
  IF osResult <> osm_SocketOk THEN Exit;
{$IFDEF Windows}
  pModeECB^.Socket := ASocket;
{$ELSE}
  AnECB^.Socket := ASocket;
{$ENDIF}
  SendPacketWithSocket(AnECB);
  RelinquishControl;
  CloseSocket(ASocket)
END;

PROCEDURE ListenForPacket(AnECB: pECB);
{$IFDEF Windows}
BEGIN
  FillStruc(ListenStrc,AnECB, 0, 4, 0, 0);
  RMInt(ListenStrc);

{$ELSE}
                                                  ASSEMBLER;
ASM
  LES  SI, AnECB
  MOV  BX, 4                            (* Funktion $04   *)
  CALL DWORD Ptr IPX_Adr                (* Keine Rckgabe *)
{$ENDIF}
END;

FUNCTION CancelEvent(AnECB: pECB): BYTE;
{$IFDEF Windows}
BEGIN
  FillStruc(CallStrc, AnECB, 0, 6, 0, 0);
  RMInt(CallStrc)

{$ELSE}
                                                  ASSEMBLER;
ASM
  LES  SI, AnECB
  MOV  BX, 6                              (* Funktion $06 *)
  CALL DWORD Ptr IPX_Adr
{$ENDIF}    (* Ergebnis in AL:  0 = erfolgreich           *)
END;        (*                $F9 = Lschen nicht mglich *)
            (*                $FF = ECB existiert nicht   *)

PROCEDURE RelinquishControl;
{$IFDEF Windows}
BEGIN
  FillStruc(CallStrc, NIL, 0, $0A, 0, 0);
  RMInt(CallStrc)

{$ELSE}
                                                  ASSEMBLER;
ASM
  MOV  BX, 0AH                          (* Funktion $0A   *)
  CALL DWORD Ptr IPX_Adr                (* Keine Rckgabe *)
{$ENDIF}
END;

END.

(*                     Ende von IPX.PAS                   *)
(*========================================================*)
